<?php
/**
 * Forty_Two_Sticky_Elements class
 *
 * @package Forty_Two
 * @license GPL-2.0-or-later
 * @link    https://aspengrovestudios.com/themes/forty-two/
 */

/**
 * Class responsible for managing sticky frontend elements.
 *
 * @since 1.0.0
 */
class Forty_Two_Sticky_Elements extends Forty_Two_Theme_Component_Base {

	/**
	 * Registered theme sticky elements.
	 *
	 * @since 1.0.0
	 * @var array
	 */
	private $sticky_elements = array();

	/**
	 * Constructor.
	 *
	 * Sets the required dependencies.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		$this->require_dependency_class( 'Forty_Two_Settings' );
		$this->require_dependency_class( 'Forty_Two_Assets' );
		$this->require_dependency_class( 'Forty_Two_Customizer' );
	}

	/**
	 * Registers a sticky theme frontend element.
	 *
	 * @since 1.0.0
	 *
	 * @param Forty_Two_Sticky_Element $sticky Sticky element to register.
	 * @return bool True on success, false on failure.
	 */
	public function register_sticky_element( Forty_Two_Sticky_Element $sticky ) {
		$id = $sticky->get_prop( Forty_Two_Sticky_Element::PROP_ID );

		if ( isset( $this->sticky_elements[ $id ] ) ) {
			return false;
		}

		$this->sticky_elements[ $id ] = $sticky;

		return true;
	}

	/**
	 * Gets a registered sticky theme frontend element.
	 *
	 * @since 1.0.0
	 *
	 * @param string $id Unique string identifier for this setting.
	 * @return Forty_Two_Sticky_Element Registered setting instance.
	 *
	 * @throws Forty_Two_Sticky_Element_Not_Registered_Exception Thrown when $id does not identify a registered setting.
	 */
	public function get_registered_sticky_element( $id ) {
		if ( ! isset( $this->sticky_elements[ $id ] ) ) {
			throw Forty_Two_Sticky_Element_Not_Registered_Exception::from_id( $id );
		}

		return $this->sticky_elements[ $id ];
	}

	/**
	 * Gets all registered sticky theme frontend elements.
	 *
	 * @since 1.0.0
	 *
	 * @return array Array of $key => $sticky pairs, where each $sticky is a
	 *               registered Forty_Two_Sticky_Element instance.
	 */
	public function get_registered_sticky_elements() {
		return $this->sticky_elements;
	}

	/**
	 * Magic call method.
	 *
	 * Handles the widgets registration action hook callbacks.
	 *
	 * @since 1.0.0
	 *
	 * @param string $method Method name.
	 * @param array  $args   Method arguments.
	 *
	 * @throws BadMethodCallException Thrown when method name is invalid.
	 */
	public function __call( $method, $args ) {
		switch ( $method ) {
			case 'register_settings':
			case 'register_customize_controls_js':
			case 'register_customize_preview_js':
			case 'add_main_script_data':
				return call_user_func_array( array( $this, $method ), $args );
			default:
				/* translators: %s: method name */
				throw new BadMethodCallException( sprintf( __( 'Call to undefined method %s', 'forty-two' ), __CLASS__ . '::' . $method . '()' ) );
		}
	}

	/**
	 * Registers settings for navbar behavior.
	 *
	 * @since 1.0.0
	 */
	protected function register_settings() {
		$settings = $this->get_dependency( 'settings' );

		foreach ( $this->sticky_elements as $sticky ) {
			$settings->register_setting( $sticky->get_setting() );
		}
	}

	/**
	 * Registers scripts for the Customizer controls.
	 *
	 * @since 1.0.0
	 *
	 * @param Forty_Two_Assets $assets Assets instance.
	 */
	protected function register_customize_controls_js( $assets ) {
		$data = array(
			'stickyElements' => array(),
		);

		foreach ( $this->sticky_elements as $id => $sticky ) {
			$props            = $sticky->get_props();
			$props['setting'] = $sticky->get_setting()->get_prop( Forty_Two_Setting::PROP_ID );

			$data['stickyElements'][] = $props;
		}

		$assets->register_asset( new Forty_Two_Script(
			'forty-two-sticky-elements-customize-controls',
			get_theme_file_uri( '/assets/dist/js/sticky-elements.customize-controls.js' ),
			array(
				Forty_Two_Script::PROP_DEPENDENCIES => array( 'customize-controls', 'wp-i18n' ),
				Forty_Two_Script::PROP_VERSION      => FORTY_TWO_VERSION,
				Forty_Two_Script::PROP_LOCATION     => Forty_Two_Script::LOCATION_CUSTOMIZE_CONTROLS,
				Forty_Two_Script::PROP_MIN_URI      => true,
				Forty_Two_Script::PROP_DATA_NAME    => 'themeStickyElementsControlsData',
				Forty_Two_Script::PROP_DATA         => $data,
			)
		) );
	}

	/**
	 * Registers scripts for the Customizer preview.
	 *
	 * @since 1.0.0
	 *
	 * @param Forty_Two_Assets $assets Assets instance.
	 */
	protected function register_customize_preview_js( $assets ) {
		$data = array(
			'stickyElements' => array(),
		);

		foreach ( $this->sticky_elements as $id => $sticky ) {
			$data['stickyElements'][] = array(
				'setting'  => $sticky->get_setting()->get_prop( Forty_Two_Setting::PROP_ID ),
				'selector' => $sticky->get_prop( Forty_Two_Sticky_Element::PROP_SELECTOR ),
				'location' => $sticky->get_prop( Forty_Two_Sticky_Element::PROP_LOCATION ),
			);
		}

		$assets->register_asset( new Forty_Two_Script(
			'forty-two-sticky-elements-customize-preview',
			get_theme_file_uri( '/assets/dist/js/sticky-elements.customize-preview.js' ),
			array(
				Forty_Two_Script::PROP_DEPENDENCIES => array( 'customize-preview' ),
				Forty_Two_Script::PROP_VERSION      => FORTY_TWO_VERSION,
				Forty_Two_Script::PROP_LOCATION     => Forty_Two_Script::LOCATION_CUSTOMIZE_PREVIEW,
				Forty_Two_Script::PROP_MIN_URI      => true,
				Forty_Two_Script::PROP_DATA_NAME    => 'themeStickyElementsPreviewData',
				Forty_Two_Script::PROP_DATA         => $data,
			)
		) );
	}

	/**
	 * Adds script data for navigation functionality.
	 *
	 * @since 1.0.0
	 */
	protected function add_main_script_data() {
		$assets = $this->get_dependency( 'assets' );

		$stickies = array();
		foreach ( $this->sticky_elements as $sticky ) {
			if ( ! $sticky->get_setting()->get_value() ) {
				continue;
			}

			$selector = $sticky->get_prop( Forty_Two_Sticky_Element::PROP_SELECTOR );
			$location = $sticky->get_prop( Forty_Two_Sticky_Element::PROP_LOCATION );

			$stickies[ $selector ] = $location;
		}

		$script = $assets->get_registered_asset( 'forty-two-script' );
		$script->add_data( 'sticky', $stickies );
	}

	/**
	 * Adds hooks and runs other processes required to initialize the component.
	 *
	 * @since 1.0.0
	 */
	protected function run_initialization() {
		add_action( 'after_setup_theme', array( $this, 'register_settings' ), 100, 0 );
		add_action( 'wp_head', array( $this, 'add_main_script_data' ), 0, 0 );

		$customizer = $this->get_dependency( 'customizer' );
		$customizer->on_js_controls_init( array( $this, 'register_customize_controls_js' ) );
		$customizer->on_js_preview_init( array( $this, 'register_customize_preview_js' ) );
	}
}
